"""
Configs migration

Generation datetime: 2025-02-20 21:30:36.332890+00:00
"""

import ujson
from configs_core.migration import models
from configs_core.migration.db_migrate_helper import (
    DatabaseMigrationHelper,
    ExternalLimitationToModify,
    LimitationToModify,
)
from sqlalchemy import and_, outerjoin, select, update

mh: DatabaseMigrationHelper

REVISION = "af3eb108"
DOWN_REVISION = "e1c45c74"


def downgrade(oldValue: dict):
    oldValue.pop("max_active_tasks", None)
    return oldValue


limitationsToModify: list[LimitationToModify] = [
    LimitationToModify(
        limitationName="LUNA_IMAGE_STORE_OBJECTS_TIMEOUTS",
        settingUpgradeFunction=lambda oldValue: oldValue,
        settingDowngradeFunction=lambda oldValue: oldValue,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-api"],
        newServices=["luna-api", "luna-video-agent"],
    ),
    LimitationToModify(
        limitationName="LUNA_VIDEO_AGENT_SETTINGS",
        oldValidationSchema={
            "type": "object",
            "properties": {"max_active_streams": {"type": "integer", "minimum": 1}},
            "description": "extended video agent settings",
            "additionalProperties": False,
        },
        newValidationSchema={
            "type": "object",
            "properties": {
                "max_active_streams": {"type": "integer", "minimum": 1},
                "max_active_tasks": {"type": "integer", "minimum": 1},
            },
            "description": "extended video agent settings",
            "additionalProperties": False,
        },
        oldDefaultValue={"max_active_streams": 2},
        newDefaultValue={"max_active_streams": 2, "max_active_tasks": 2},
        settingUpgradeFunction=lambda oldValue: oldValue | {"max_active_tasks": 2},
        settingDowngradeFunction=downgrade,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-video-agent"],
        newServices=["luna-video-agent"],
    ),
]

externalLimitationsToModify: list[ExternalLimitationToModify] = []


def getSetting(name: str) -> list:
    """
    Get setting by name from db.
    Args:
        name: setting name
    Returns:
        settings list
    """
    dbReply = mh.connection.execute(
        select(
            [models.Tag.tag, models.Setting.value, models.Setting.description],
            from_obj=outerjoin(models.Setting, models.Tag, models.Setting.id == models.Tag.id),
        ).where(models.Setting.name == name)
    ).cursor.fetchall()
    return [(tag, ujson.loads(value), description) for tag, value, description in dbReply]


def setSetting(value: dict, name: str):
    """
    Set setting by name.
    Args:
        value: settings to be set
        name: luna-handlers setting name
    """
    mh.connection.execute(
        update(models.Setting)
        .where(and_(models.Setting.name == name, models.Setting.is_default == True))
        .values(value=ujson.dumps(value))
    )


def upgrade():
    """Upgrade configs to new revision"""

    # limitations to add
    mh.addLimitation(
        limitationName="LUNA_VIDEO_AGENT_REDIS_DB_ADDRESS",
        validationSchema={
            "type": "object",
            "properties": {
                "host": {"type": "string"},
                "port": {"type": "integer", "minimum": 1, "maximum": 65535},
                "user": {"type": "string"},
                "password": {"type": "string"},
                "sentinel": {
                    "description": "Redis sentinels settings",
                    "type": "object",
                    "properties": {
                        "master_name": {"type": "string"},
                        "sentinels": {
                            "type": "array",
                            "items": {
                                "description": "Redis sentinel settings. Host+port",
                                "type": "string",
                                "pattern": "[a-zA-Z0-9.]+:\\d{1,5}",
                                "example": "127.0.0.1:26379",
                            },
                        },
                        "user": {"type": "string"},
                        "password": {"type": "string"},
                    },
                    "additionalProperties": False,
                    "required": ["master_name", "sentinels", "user", "password"],
                },
                "number": {
                    "type": "integer",
                    "enum": [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14],
                },
            },
            "additionalProperties": False,
        },
        defaultValue={
            "host": "127.0.0.1",
            "port": 6379,
            "user": "",
            "password": "",
            "sentinel": {
                "master_name": "video_task_master",
                "sentinels": [],
                "user": "",
                "password": "",
            },
            "number": 0,
        },
        description="Redis database address",
        services=["luna-video-agent"],
        groupNames=["luna-platform"],
    )

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)

    for setting in getSetting("LUNA_ATTRIBUTES_DB"):
        if setting[0] is None:
            setSetting(setting[1], "LUNA_VIDEO_AGENT_REDIS_DB_ADDRESS")


def downgrade():
    """Downgrade configs to previous revision"""

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)

    # limitations to remove
    mh.removeLimitation(limitationName="LUNA_VIDEO_AGENT_REDIS_DB_ADDRESS")
