"""
Configs migration

Generation datetime: 2025-03-17 09:05:06.251369+00:00
"""

from configs_core.migration.db_migrate_helper import (
    DatabaseMigrationHelper,
    ExternalLimitationToModify,
    LimitationToModify,
)

mh: DatabaseMigrationHelper

REVISION = "52bdc94d"
DOWN_REVISION = "11aa1d48"


def upgradeApiVersion(value):
    value["api_version"] = 2
    return value


def downgradeApiVersion(value):
    value["api_version"] = 1
    return value


def dbSettingUpgradeFunction(oldSetting):
    if oldSetting["db_type"] == "oracle":
        mh.logger.warning(
            f"The *oracle* database support has been terminated. "
            f"The old `LUNA_VIDEO_MANAGER_DB` setting will be replaced with the new one. The old setting: {oldSetting}"
        )
        return {
            "db_type": "postgres",
            "db_user": "luna",
            "db_password": "luna",
            "db_name": "luna_video_manager",
            "db_host": "127.0.0.1",
            "db_port": 5432,
            "db_settings": {"connection_pool_size": 10},
        }
    return oldSetting


limitationsToModify: list[LimitationToModify] = [
    LimitationToModify(
        limitationName="LUNA_VIDEO_MANAGER_DB",
        oldValidationSchema={
            "properties": {
                "db_type": {"type": "string", "enum": ["postgres", "oracle"]},
                "dsn": {
                    "type": "string",
                    "pattern": "^(?:(?P<user>.*?)(?::(?P<passwd>.*))@)?(?P<net>[^\\?\\/]*)(?:\\/(?P<dbname>[^\\?]*)(?:\\?(?P<params>[^\\?]*))?)?$",
                },
                "db_user": {"type": "string"},
                "db_password": {"type": "string"},
                "db_name": {"type": "string"},
                "db_host": {"type": "string"},
                "db_port": {"type": "integer", "minimum": 1, "maximum": 65535},
                "db_settings": {
                    "type": "object",
                    "properties": {"connection_pool_size": {"type": "integer", "minimum": 1}},
                    "required": ["connection_pool_size"],
                    "additionalProperties": False,
                },
            },
            "required": ["db_type", "db_settings"],
            "type": "object",
            "additionalProperties": False,
        },
        newValidationSchema={
            "properties": {
                "db_type": {"type": "string", "enum": ["postgres"]},
                "dsn": {
                    "type": "string",
                    "pattern": "^(?:(?P<user>.*?)(?::(?P<passwd>.*))@)?(?P<net>[^\\?\\/]*)(?:\\/(?P<dbname>[^\\?]*)(?:\\?(?P<params>[^\\?]*))?)?$",
                },
                "db_user": {"type": "string"},
                "db_password": {"type": "string"},
                "db_name": {"type": "string"},
                "db_host": {"type": "string"},
                "db_port": {"type": "integer", "minimum": 1, "maximum": 65535},
                "db_settings": {
                    "type": "object",
                    "properties": {"connection_pool_size": {"type": "integer", "minimum": 1}},
                    "required": ["connection_pool_size"],
                    "additionalProperties": False,
                },
            },
            "required": ["db_type", "db_settings"],
            "type": "object",
            "additionalProperties": False,
        },
        settingUpgradeFunction=dbSettingUpgradeFunction,
        settingDowngradeFunction=lambda oldValue: oldValue,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-video-manager"],
        newServices=["luna-video-manager"],
    ),
    LimitationToModify(
        limitationName="LUNA_VIDEO_MANAGER_ADDRESS",
        oldDefaultValue={"origin": "http://127.0.0.1:5230", "api_version": 1},
        newDefaultValue={"origin": "http://127.0.0.1:5230", "api_version": 2},
        settingUpgradeFunction=upgradeApiVersion,
        settingDowngradeFunction=downgradeApiVersion,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-admin", "luna-api", "luna-video-agent"],
        newServices=["luna-admin", "luna-api", "luna-video-agent"],
    ),
    LimitationToModify(
        limitationName="LUNA_VIDEO_AGENT_ADDRESS",
        oldDefaultValue={"origin": "http://127.0.0.1:5240", "api_version": 1},
        newDefaultValue={"origin": "http://127.0.0.1:5240", "api_version": 2},
        settingUpgradeFunction=upgradeApiVersion,
        settingDowngradeFunction=downgradeApiVersion,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=[
            "luna-admin",
            "luna-api",
            "luna-lambda-tasks-unit",
            "luna-lambda-unit",
        ],
        newServices=[
            "luna-admin",
            "luna-api",
            "luna-lambda-tasks-unit",
            "luna-lambda-unit",
        ],
    ),
]

externalLimitationsToModify: list[ExternalLimitationToModify] = []


def upgrade():
    """Upgrade configs to new revision"""

    # limitations to add
    mh.addLimitation(
        limitationName="LUNA_VIDEO_MANAGER_LOGS_CLEAR_INTERVAL",
        validationSchema={
            "properties": {
                "interval": {
                    "type": "integer",
                    "minimum": 1,
                    "description": "Delete logs older than interval.",
                },
                "interval_type": {
                    "type": "string",
                    "enum": ["weeks", "days", "hours", "minutes", "seconds"],
                },
                "check_interval": {
                    "type": "integer",
                    "minimum": 0,
                    "description": "Logs check interval in seconds for deletion.",
                },
                "active": {
                    "type": "boolean",
                    "description": "Activate streams logs auto deletion.",
                },
            },
            "required": ["interval", "interval_type", "check_interval", "active"],
            "type": "object",
            "additionalProperties": False,
        },
        defaultValue={
            "interval": 7,
            "interval_type": "days",
            "check_interval": 180,
            "active": True,
        },
        description="Interval for clear logs.",
        services=["luna-video-manager"],
        groupNames=["luna-platform"],
    )

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)


def downgrade():
    """Downgrade configs to previous revision"""

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)

    # limitations to remove
    mh.removeLimitation(limitationName="LUNA_VIDEO_MANAGER_LOGS_CLEAR_INTERVAL")
