from typing import Any, final

from cow.errors.exception import VLException
from cow.pydantic.loaders import loadDataFromJson
from cow.web.request import VLRequest
from luna3.client import Client as _Client
from sanic import Sanic
from vlutils.structures.pydantic import BaseModel

from luna_lambda_tools.public.clients.events import Events
from luna_lambda_tools.public.clients.faces import Faces
from luna_lambda_tools.public.clients.handlers import Handlers
from luna_lambda_tools.public.clients.lis import LIS
from luna_lambda_tools.public.clients.lpm import LPM
from luna_lambda_tools.public.clients.lrs import LRS
from luna_lambda_tools.public.clients.lva import LVA
from luna_lambda_tools.public.clients.sender import Sender

from ..public.clients import Clients
from .base_application import BaseAppCtx
from .exceptions import BaseLambdaException
from .monitoring import MonitoringPoint


class BaseLambdaRequest(VLRequest[BaseAppCtx]):
    """
    Luna-configurator request
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.accountId = self.headers.get("Luna-Account-Id")
        self.userCtx = self.app.ctx.userCtx

    @property
    def json(self) -> int | float | list | dict[str, Any]:
        """Get json from request"""
        return super().json

    def getBody(self) -> bytes:
        """Get body from request"""
        return super().body

    def sendToMonitoring(self, points: list[MonitoringPoint]) -> None:
        """
        Send point to monitoring

        Args:
            points: points
        """
        if self.app.ctx.serviceConfig.monitoring.sendData:
            self.app.ctx.monitoring.flushPoints(points)


class ClientsKwargsMixin:
    """Luna services clients kwargs mixin"""

    _luna3Client: _Client
    accountId: str
    app: Sanic

    @property
    def _session(self):
        if hasattr(self.app, "ctx"):
            return self.app.ctx.luna3session
        return self.app.luna3session

    @property
    def _serviceConfig(self):
        if hasattr(self.app, "ctx"):
            return self.app.ctx.serviceConfig
        return self.app.serviceConfig

    @property
    def facesEnabled(self) -> bool:
        """Whether luna-faces is enabled"""
        return self._session.facesSession is not None

    def _getFacesClientKwargs(self) -> dict:
        """Get faces clients kwargs"""
        if self.facesEnabled:
            return {
                "facesClient": self._session.facesSession.getClient(self.requestId),
                "accountId": self.accountId,
            }

    @property
    def eventsEnabled(self) -> bool:
        """Whether luna-events is enabled"""
        return self._session.eventSession is not None

    def _getEventsClientKwargs(self) -> dict | None:
        """Get events clients kwargs"""
        if self.eventsEnabled:
            return {
                "eventsClient": self._session.eventSession.getClient(self.requestId),
                "accountId": self.accountId,
            }

    @property
    def matcherEnabled(self) -> bool:
        """Whether lpm is enabled"""
        return self._session.pythonMatcherSession is not None

    def _getLPMClientKwargs(self) -> dict:
        """Get lpm clients kwargs"""
        if self.matcherEnabled:
            return {
                "lpmClient": self._session.pythonMatcherSession.getClient(self.requestId),
                "accountId": self.accountId,
            }

    @property
    def lvaEnabled(self) -> bool:
        """Whether Luna-Video-Agent is enabled"""
        return self._session.videoAgentSession is not None

    def _getLVAClientKwargs(self) -> dict | None:
        """Get lpn clients kwargs"""
        if self.lvaEnabled:
            return {
                "lvaClient": self._session.videoAgentSession.getClient(self.requestId),
                "accountId": self.accountId,
            }

    @property
    def faceSampleStoreEnabled(self) -> bool:
        """Whether face sample store is enabled"""
        return self._session.faceSampleStoreSession is not None

    def _getFacesSamplesClientKwargs(self) -> dict | None:
        """Get faces samples clients kwargs"""
        if self.faceSampleStoreEnabled:
            return {
                "lisClient": self._session.faceSampleStoreSession.getClient(self.requestId),
                "accountId": self.accountId,
                "bucket": self._serviceConfig.faceSamplesStorage.bucket,
            }

    @property
    def bodySampleStoreEnabled(self) -> bool:
        """Whether body sample store is enabled"""
        return self._session.bodySampleStoreSession is not None

    def _getBodySamplesClientKwargs(self) -> dict | None:
        """Get body samples clients kwargs"""
        if self.bodySampleStoreEnabled:
            return {
                "lisClient": self._session.bodySampleStoreSession.getClient(self.requestId),
                "accountId": self.accountId,
                "bucket": self._serviceConfig.bodySamplesStorage.bucket,
            }

    @property
    def imageOriginStoreEnabled(self) -> bool:
        """Whether image origin store is enabled"""
        return self._session.imageOriginStoreSession is not None

    def _getImageOriginClientKwargs(self) -> dict | None:
        """Get image origin clients kwargs"""
        if self.imageOriginStoreEnabled:
            return {
                "lisClient": self._session.imageOriginStoreSession.getClient(self.requestId),
                "accountId": self.accountId,
                "bucket": self._serviceConfig.imageOriginStorage.bucket,
            }

    @property
    def remoteSDKEnabled(self) -> bool:
        """Whether remote sdk is enabled"""
        return self._session.remoteSDKSession is not None

    def _getLRSClientKwargs(self) -> dict | None:
        """Get lrs clients kwargs"""
        if self.remoteSDKEnabled:
            return {
                "lrsClient": self._session.remoteSDKSession.getClient(self.requestId),
                "accountId": self.accountId,
            }

    @property
    def senderEnabled(self) -> bool:
        """Whether sender is enabled"""
        return self._session.senderSession is not None

    def _getSenderClientKwargs(self) -> dict | None:
        """Get sender clients kwargs"""
        if self.senderEnabled:
            return {
                "senderClient": self._session.senderSession.getClient(self.requestId),
                "accountId": self.accountId,
            }

    @property
    def handlersEnabled(self) -> bool:
        """Whether handlers is enabled"""
        return self._session.handlersSession is not None

    def _getHandlersClientKwargs(self) -> dict | None:
        """Get handlers clients kwargs"""
        if self.handlersEnabled:
            return {
                "handlersClient": self._session.handlersSession.getClient(self.requestId),
                "accountId": self.accountId,
            }


class BaseStandaloneLambdaRequest(BaseLambdaRequest, ClientsKwargsMixin):
    """Base lambda request"""

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._luna3Client: _Client = self.app.ctx.luna3session.getClient(self.requestId)
        self._clients: Clients | None = None

    @property
    def clients(self):
        if self._clients is None:
            facesKw = self._getFacesClientKwargs()
            lpmKw = self._getLPMClientKwargs()
            eventsKw = self._getEventsClientKwargs()
            faceSamplesKw = self._getFacesSamplesClientKwargs()
            bodySamplesKw = self._getBodySamplesClientKwargs()
            imageOriginKw = self._getImageOriginClientKwargs()
            lvaKw = self._getLVAClientKwargs()
            senderKw = self._getSenderClientKwargs()
            handlersKw = self._getHandlersClientKwargs()
            lrsKw = self._getLRSClientKwargs()
            self._clients = Clients(
                faces=Faces(**facesKw) if facesKw else None,
                events=Events(**eventsKw) if eventsKw else None,
                matcher=LPM(**lpmKw) if lpmKw else None,
                faceSamplesStore=LIS(**faceSamplesKw) if faceSamplesKw else None,
                bodySamplesStore=LIS(**bodySamplesKw) if bodySamplesKw else None,
                imageOriginStore=LIS(**imageOriginKw) if imageOriginKw else None,
                sdk=LRS(**lrsKw) if lrsKw else None,
                videoAgent=LVA(**lvaKw) if lvaKw else None,
                sender=Sender(**senderKw) if senderKw else None,
                handlers=Handlers(**handlersKw) if handlersKw else None,
            )
        return self._clients


class BaseHandlersRequest(BaseStandaloneLambdaRequest):
    """Base lambda request"""

    @final
    def getRequestData(self, pydanticModel: type[BaseModel]):
        """Get data from request"""
        try:
            return loadDataFromJson(super().json, pydanticModel)
        except VLException as e:
            exc = BaseLambdaException
            exc.errorText = str(e.error.asDict())
            exc.statusCode = 400
            raise exc
